# install.packages('caTools')
library(tidyverse)
library(magrittr)
library(caTools)
library(caret)
library(class)
library(dplyr)
library(e1071)
library(rpart)
library(randomForest)


#The dataset comes from here: https://www.metabolomicsworkbench.org/data/DRCCMetadata.php?Mode=Study&DataMode=AllData&StudyID=ST000007&StudyType=MS&ResultType=1#DataTabs


# Importing the dataset
dataset = read_delim('ST000007.csv',
                     delim = ",",
                     col_names = TRUE,
                     trim_ws = TRUE)
#the varaibles and observetions are in the wrong order in the dataset
dataset <- gather(dataset, key = "local_sample_id", value = "Signal", 2:61)

dataset <- spread(dataset, key = "Name", value = "Signal")

metadata = read_delim('metadata.csv',
                     delim = ",",
                     col_names = TRUE,
                     trim_ws = TRUE)

metadata <- metadata %>%
  mutate(local_sample_id = as.character(local_sample_id))

dataset <- dataset %>%
  left_join(metadata) %>%
  filter(genotype != "Taipei 309") %>% #lets model one cell type at a time
  select(-c(mb_sample_id, local_sample_id, genotype)) #remove the columns that are not the features

dataset <- dataset %>%
  filter(dataset$'Xoo infection' != "mock",
         dataset$'Xoo infection' != "no treatment") 

# Encoding the target feature as factor
dataset <- dataset %>%
  mutate('Xoo infection' = factor(dataset$`Xoo infection`))%>%
  select('Xoo infection', everything())



#data cleaning, remove variables that do not change from sample to sample and the ones that are strongly correlated
dataset2 <- dataset %>% 
  select(-'Xoo infection')

dataset2 <- dataset2 %>%
  select(-nearZeroVar(dataset2)) %>%
  select(-findCorrelation(cor(dataset2), cutoff = 0.6))%>%
  mutate('Xoo infection' = dataset$`Xoo infection`) %>%
  select('Xoo infection', everything())

# Feature Scaling
dataset2[, 2:52] = scale(dataset2[, 2:52])
dataset <- dataset2

# Splitting the dataset into the Training set and Test set
set.seed(123)
split = sample.split(dataset$'Xoo infection', SplitRatio = 0.75)
training_set = subset(dataset, split == TRUE)
test_set = subset(dataset, split == FALSE)



#-------------------------------logistic regression----------------------------------



fitControl <- trainControl(## 10-fold CV
  method = "repeatedcv",
  number = 2,
  ## repeated ten times
  repeats = 1)

set.seed(825)
classifier <- train(`Xoo infection` ~ ., 
                    data = training_set, 
                   method = "glmStepAIC", 
                   trControl = fitControl,
                   verbose = FALSE)

summary(classifier)

# Predicting the Test set results
training_set <- training_set %>%n 
  mutate(Class_pred = predict(classifier, newdata = training_set, type = 'raw')) %>%
  select('Xoo infection', Class_pred, everything())

test_set <- test_set %>%
  mutate(Class_pred = predict(classifier, newdata = test_set, , type = 'raw')) %>%
  select('Xoo infection', Class_pred, everything())



#----------------------------------KNN------------------------------------------------------

fitControl <- trainControl(## 10-fold CV
  method = "repeatedcv",
  number = 2,
  ## repeated ten times
  repeats = 1)

set.seed(825)
classifier <- train(`Xoo infection` ~ ., 
                    data = training_set, 
                    method = "knn", 
                    trControl = fitControl,
                    verbose = FALSE)

summary(classifier)
classifier

# Predicting the Test set results
training_set <- training_set %>%
  mutate(Class_pred = predict(classifier, newdata = training_set, type = 'raw')) %>%
  select('Xoo infection', Class_pred, everything())

test_set <- test_set %>%
  mutate(Class_pred = predict(classifier, newdata = test_set, , type = 'raw')) %>%
  select('Xoo infection', Class_pred, everything())


#-------------------------------------------------LDA---------------------------------------------------------

fitControl <- trainControl(## 10-fold CV
  method = "repeatedcv",
  number = 2,
  ## repeated ten times
  repeats = 1)

set.seed(825)
classifier <- train(`Xoo infection` ~ ., 
                    data = training_set, 
                    method = "stepLDA", 
                    trControl = fitControl,
                    verbose = FALSE)

summary(classifier)
classifier
classifier$finalModel

model <- MASS::lda(`Xoo infection` ~ ., 
                 data = training_set)


# Predicting the Test set results
training_set <- training_set %>%
  mutate(Class_pred = predict(classifier, newdata = training_set, type = 'raw')) %>%
  select('Xoo infection', Class_pred, everything())

test_set <- test_set %>%
  mutate(Class_pred = predict(classifier, newdata = test_set, , type = 'raw')) %>%
  select('Xoo infection', Class_pred, everything())



#-------------------------------------------decision tree------------------------------------------------


# Feature Scaling in actually not required with the Decision Trees approach, but it also does not do any harm
#you should test applying feature scaling and not applying it

colnames(training_set) <- make.names(colnames(training_set))
colnames(test_set) <- make.names(colnames(test_set))

fitControl <- trainControl(## 10-fold CV
  method = "repeatedcv",
  number = 2,
  ## repeated ten times
  repeats = 1)

set.seed(825)
classifier <- train(Xoo.infection ~ ., 
                    data = training_set, 
                    method = "rpart", 
                    trControl = fitControl)

summary(classifier)
classifier


# classifier = rpart(formula = training_set$`Xoo infection` ~ .,
#                    data = training_set)
# 
rpart.plot(classifier, box.palette="RdBu", shadow.col="gray", nn=TRUE)

# Predicting the Test set results
training_set <- training_set %>%
  mutate(Class_pred = predict(classifier, newdata = training_set)) %>%
  select(Xoo.infection, Class_pred, everything())

test_set <- test_set %>%
  mutate(Class_pred = predict(classifier, newdata = test_set)) %>%
  select(Xoo.infection, Class_pred, everything())


#------------------------------------------Random Forest------------------------------------------------

# Feature Scaling is again not needed
# Fitting Random Forest Classification to the Training set
colnames(training_set) <- make.names(colnames(training_set))
colnames(test_set) <- make.names(colnames(test_set))

fitControl <- trainControl(## 10-fold CV
  method = "repeatedcv",
  number = 2,
  ## repeated ten times
  repeats = 1)

set.seed(825)
classifier <- train(Xoo.infection ~ ., 
                    data = training_set, 
                    method = "rf", 
                    trControl = fitControl)

classifier
summary(classifier)


# Predicting the Test set results
training_set <- training_set %>%
  mutate(Class_pred = predict(classifier, newdata = training_set)) %>%
  select(Xoo.infection, Class_pred, everything())

test_set <- test_set %>%
  mutate(Class_pred = predict(classifier, newdata = test_set)) %>%
  select(Xoo.infection, Class_pred, everything())

confusionMatrix(test_set$Xoo.infection, test_set$Class_pred)